@@ -18,6 +18,7 @@ under the License.
 '''
 
 import os
+import stat
 import re
 import glob
 import resource
@@ -96,8 +97,8 @@ class StorageObject(CFSNode):
             fwrite(self.path + "/pr/res_aptpl_metadata", ",".join(res))
 
     @classmethod
-    def all(cls, path):
-        for so_dir in glob.glob("%s/core/*_*/*" % path):
+    def all(cls):
+        for so_dir in glob.glob("%s/core/*_*/*" % cls.configfs_dir):
             if os.path.isdir(so_dir):
                 yield cls.so_from_path(so_dir)
 
@@ -740,6 +741,25 @@ class BlockStorageObject(StorageObject):
         return d
 
 
+class StorageObjectFactory(object):
+    """
+    Create a storage object based on a given path.
+    Only works for file & block.
+    """
+
+    def __new__(cls, path):
+        path = os.path.realpath(path)
+        name = path.strip("/").replace("/", "-")
+        if os.path.exists(path):
+            s = os.stat(path)
+            if stat.S_ISBLK(s.st_mode):
+                return BlockStorageObject(name=name, dev=path)
+            elif stat.S_ISREG(s.st_mode):
+                return FileIOStorageObject(name=name, dev=path, size=s.st_size)
+
+        raise RTSLibError("Can't create storageobject from path: %s" % path)
+
+
 bs_params = {
     PSCSIStorageObject: dict(name='pscsi'),
     RDMCPStorageObject: dict(name='ramdisk', alt_dirprefix='rd_mcp'),
@@ -763,34 +783,35 @@ class _Backstore(CFSNode):
 
         dirp = bs_params[self._so_cls].get("alt_dirprefix", self._plugin)
 
-        # mapping in cache?
-        self._lookup_key = "%s/%s" % (dirp, name)
-        self._index = bs_cache.get(self._lookup_key, None)
-
-        if self._index == None:
-            # cache miss. Check the fs and load cache for next time
+        global bs_cache
+        if not bs_cache:
             for dir in glob.iglob("%s/core/*_*/*/" % self.configfs_dir):
                 parts = dir.split("/")
                 bs_name = parts[-2]
                 bs_dirp, bs_index = parts[-3].rsplit("_", 1)
                 current_key = "%s/%s" % (bs_dirp, bs_name)
-                if current_key == self._lookup_key:
-                    if mode == 'create':
-                        raise RTSLibError("Storage object %s/%s exists" %
-                                          (self._plugin, name))
-                    else:
-                        self._index = int(bs_index)
                 bs_cache[current_key] = int(bs_index)
 
-        if self._index == None:
-            # Allocate new index value
-            for index in xrange(1048576):
-                if index not in bs_cache.values():
-                    self._index = index
-                    bs_cache[self._lookup_key] = self._index
-                    break
+        # mapping in cache?
+        self._lookup_key = "%s/%s" % (dirp, name)
+        self._index = bs_cache.get(self._lookup_key, None)
+
+        if self._index != None and mode == 'create':
+            raise RTSLibError("Storage object %s/%s exists" %
+                              (self._plugin, name))
+        elif self._index == None:
+            if mode == 'lookup':
+                raise RTSLibError("Storage object %s/%s not found" %
+                                  (self._plugin, name))
             else:
-                raise RTSLibError("No available backstore index")
+                # Allocate new index value
+                for index in xrange(1048576):
+                    if index not in bs_cache.values():
+                        self._index = index
+                        bs_cache[self._lookup_key] = self._index
+                        break
+                else:
+                    raise RTSLibError("No available backstore index")
 
         self._path = "%s/core/%s_%d" % (self.configfs_dir,
                                         dirp,
